"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var pretur_redux_1 = require("pretur.redux");
var Querier_1 = require("./query/Querier");
var UniqueReducible_1 = require("./UniqueReducible");
var actions_1 = require("./actions");
var Reactive = (function (_super) {
    __extends(Reactive, _super);
    function Reactive(set, query, buildFetcher, autoRefreshDebounceTime) {
        if (buildFetcher === void 0) { buildFetcher = null; }
        if (autoRefreshDebounceTime === void 0) { autoRefreshDebounceTime = 300; }
        var _this = _super.call(this) || this;
        if (set === null) {
            return _this;
        }
        _this.reactiveSet = set;
        _this.reactiveQuerier = new Querier_1.default(query);
        _this.reactviceInitialized = false;
        _this.buildFetcher = buildFetcher;
        _this.autoRefreshDebounceTime = autoRefreshDebounceTime;
        return _this;
    }
    Object.defineProperty(Reactive.prototype, "set", {
        get: function () {
            return this.reactiveSet;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Reactive.prototype, "querier", {
        get: function () {
            return this.reactiveQuerier;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Reactive.prototype, "initialized", {
        get: function () {
            return this.reactviceInitialized;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Reactive.prototype, "count", {
        get: function () {
            return this.reactiveCount;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Reactive.prototype, "valid", {
        get: function () {
            return this.reactiveSet.valid;
        },
        enumerable: true,
        configurable: true
    });
    Reactive.prototype.reduce = function (action) {
        var _this = this;
        if (actions_1.CLAY_REACTIVE_REFRESH.is(this.uniqueId, action)) {
            if (!this.buildFetcher) {
                throw new Error('buildFetcher must be provided');
            }
            pretur_redux_1.emit(action, function (dispatch) {
                if (!_this.buildFetcher) {
                    throw new Error('buildFetcher must be provided');
                }
                var fetcher = _this.buildFetcher();
                _this.refresh(dispatch, fetcher);
                fetcher.fetch();
            });
            return this;
        }
        if (actions_1.CLAY_REACTIVE_SET_INITIALIZED.is(this.uniqueId, action)) {
            if (this.reactviceInitialized) {
                return this;
            }
            var clone = this.clone();
            clone.reactviceInitialized = true;
            return clone;
        }
        if (actions_1.CLAY_REACTIVE_SET_COUNT.is(this.uniqueId, action)) {
            if (this.reactiveCount === action.payload) {
                return this;
            }
            var clone = this.clone();
            clone.reactiveCount = +action.payload;
            return clone;
        }
        var newSet = this.reactiveSet.reduce(action);
        var newQuerier = this.reactiveQuerier.reduce(action);
        if (newSet !== this.reactiveSet || newQuerier !== this.reactiveQuerier) {
            var clone_1 = this.clone();
            clone_1.reactiveSet = newSet;
            clone_1.reactiveQuerier = newQuerier;
            if (this.buildFetcher && newQuerier !== this.reactiveQuerier) {
                clearTimeout(clone_1.autoRefreshTimerId);
                var refresh = function refresh() {
                    pretur_redux_1.emit(action, function (dispatch) {
                        if (!clone_1.buildFetcher) {
                            throw new Error('buildFetcher must be provided');
                        }
                        var fetcher = clone_1.buildFetcher();
                        clone_1.refresh(dispatch, fetcher);
                        fetcher.fetch();
                    });
                };
                clone_1.autoRefreshTimerId = setTimeout(refresh, clone_1.autoRefreshDebounceTime);
            }
            return clone_1;
        }
        return this;
    };
    Reactive.prototype.init = function (dispatch, fetcher) {
        var _this = this;
        return this.reactiveSet.refresh(dispatch, this.reactiveQuerier.plain, fetcher).then(function (count) {
            if (!_this.reactviceInitialized) {
                dispatch(actions_1.CLAY_REACTIVE_SET_INITIALIZED.create.unicast(_this.uniqueId));
            }
            if (typeof count === 'number') {
                dispatch(actions_1.CLAY_REACTIVE_SET_COUNT.create.unicast(_this.uniqueId, count));
            }
        });
    };
    Reactive.prototype.initIfNeeded = function (dispatch, fetcher) {
        if (!this.reactviceInitialized) {
            this.init(dispatch, fetcher);
        }
    };
    Reactive.prototype.refresh = function (dispatch, fetcher) {
        var _this = this;
        return this.reactiveSet.refresh(dispatch, this.reactiveQuerier.plain, fetcher).then(function (count) {
            if (typeof count === 'number') {
                dispatch(actions_1.CLAY_REACTIVE_SET_COUNT.create.unicast(_this.uniqueId, count));
            }
        });
    };
    Reactive.prototype.ensureValidity = function (dispatch) {
        if (this.valid) {
            return true;
        }
        dispatch(actions_1.CLAY_DATA_DECAY.create.broadcast());
        return false;
    };
    Reactive.prototype.ensureValidityAndSync = function (dispatch, synchronizer, autoDecay, autoRefresh) {
        var _this = this;
        if (autoDecay === void 0) { autoDecay = true; }
        if (autoRefresh === void 0) { autoRefresh = true; }
        if (this.valid) {
            this.reactiveSet.appendSynchronizationModels(synchronizer);
            if (autoRefresh) {
                synchronizer.listen()
                    .then(function () { return dispatch(actions_1.CLAY_REACTIVE_REFRESH.create.unicast(_this.uniqueId)); });
            }
            return true;
        }
        if (autoDecay) {
            dispatch(actions_1.CLAY_DATA_DECAY.create.broadcast());
        }
        return false;
    };
    Reactive.prototype.skipValidityAndSync = function (dispatch, synchronizer, autoRefresh) {
        var _this = this;
        if (autoRefresh === void 0) { autoRefresh = true; }
        this.reactiveSet.appendSynchronizationModels(synchronizer);
        if (autoRefresh) {
            synchronizer.listen()
                .then(function () { return dispatch(actions_1.CLAY_REACTIVE_REFRESH.create.unicast(_this.uniqueId)); });
        }
    };
    Reactive.prototype.cloneOverride = function (clone) {
        _super.prototype.cloneOverride.call(this, clone);
        clone.reactiveSet = this.reactiveSet;
        clone.reactiveQuerier = this.reactiveQuerier;
        clone.reactviceInitialized = this.reactviceInitialized;
        clone.reactiveCount = this.reactiveCount;
        clone.buildFetcher = this.buildFetcher;
        clone.autoRefreshDebounceTime = this.autoRefreshDebounceTime;
        clone.autoRefreshTimerId = this.autoRefreshTimerId;
    };
    Reactive.prototype.createInstance = function () {
        return new Reactive(null, null);
    };
    return Reactive;
}(UniqueReducible_1.default));
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = Reactive;
