"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var immutable_1 = require("immutable");
var StatusReporter_1 = require("./StatusReporter");
var actions_1 = require("./actions");
var Set = (function (_super) {
    __extends(Set, _super);
    function Set(items, synchronized) {
        if (synchronized === void 0) { synchronized = true; }
        var _this = _super.call(this, synchronized) || this;
        _this.originalSet = _this;
        if (items) {
            _this.setItems = immutable_1.OrderedMap(items.map(function (values) {
                var newItem = _this.constructItem(true, values);
                return [newItem.uniqueId, newItem];
            }));
        }
        else {
            _this.setItems = immutable_1.OrderedMap();
        }
        return _this;
    }
    Object.defineProperty(Set.prototype, "original", {
        get: function () {
            return this.originalSet;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Set.prototype, "items", {
        get: function () {
            return this.setItems;
        },
        enumerable: true,
        configurable: true
    });
    Set.prototype.reduce = function (action) {
        var _this = this;
        if (actions_1.CLAY_DATA_CLEAR.is(this.uniqueId, action)) {
            return this.originalSet;
        }
        if (actions_1.CLAY_DATA_RESET.is(this.uniqueId, action)) {
            if (!action.payload) {
                return this.original;
            }
            var clone = this.clone();
            clone.statusInstance = this.statusInstance.setUnmodified();
            clone.setItems = immutable_1.OrderedMap(action.payload.map(function (values) {
                var newItem = _this.constructItem(true, values);
                return [newItem.uniqueId, newItem];
            }));
            clone.originalSet = clone;
            return clone;
        }
        if (actions_1.CLAY_DATA_ADD_ITEM.is(this.uniqueId, action)) {
            var clone = this.clone();
            if (this.statusInstance.synchronized) {
                clone.statusInstance = this.statusInstance.setModified();
            }
            var newItem = this.constructItem(false, action.payload);
            clone.setItems = this.setItems.set(newItem.uniqueId, newItem);
            return clone;
        }
        if (actions_1.CLAY_DATA_REMOVE_ITEM.is(this.uniqueId, action) && action.payload) {
            var newItems_1;
            var target = this.setItems.get(action.payload);
            if (target.status.added) {
                newItems_1 = this.setItems.remove(action.payload);
            }
            else {
                newItems_1 = this.setItems.set(action.payload, target.setRemoved());
            }
            if (immutable_1.is(this.originalSet.setItems, newItems_1)) {
                return this.originalSet;
            }
            var clone = this.clone();
            if (this.statusInstance.synchronized) {
                clone.statusInstance = this.statusInstance.setModified();
            }
            clone.setItems = newItems_1;
            return clone;
        }
        if (actions_1.CLAY_DATA_UNREMOVE_ITEM.is(this.uniqueId, action) && action.payload) {
            var target = this.setItems.get(action.payload);
            var newItems_2 = this.setItems.set(action.payload, target.setUnremoved());
            if (immutable_1.is(this.originalSet.setItems, newItems_2)) {
                return this.originalSet;
            }
            var clone = this.clone();
            clone.setItems = newItems_2;
            return clone;
        }
        var updated = false;
        var newItems = this.setItems.withMutations(function (i) {
            _this.setItems.forEach(function (item) {
                if (item.status.removed) {
                    return;
                }
                var newItem = item.reduce(action);
                if (newItem !== item) {
                    updated = true;
                    i.set(newItem.uniqueId, newItem);
                }
            });
        });
        if (updated) {
            if (immutable_1.is(this.originalSet.setItems, newItems)) {
                return this.originalSet;
            }
            var clone = this.clone();
            if (this.statusInstance.synchronized) {
                clone.statusInstance = this.statusInstance.setModified();
            }
            clone.setItems = newItems;
            return clone;
        }
        return this;
    };
    Set.prototype.clear = function (dispatch) {
        dispatch(actions_1.CLAY_DATA_CLEAR.create.unicast(this.uniqueId));
    };
    Set.prototype.add = function (dispatch, values) {
        dispatch(actions_1.CLAY_DATA_ADD_ITEM.create.unicast(this.uniqueId, values));
    };
    Set.prototype.remove = function (dispatch, target) {
        dispatch(actions_1.CLAY_DATA_REMOVE_ITEM.create.unicast(this.uniqueId, target));
    };
    Set.prototype.unremove = function (dispatch, target) {
        dispatch(actions_1.CLAY_DATA_UNREMOVE_ITEM.create.unicast(this.uniqueId, target));
    };
    Set.prototype.refresh = function (dispatch, query, fetcher) {
        var _this = this;
        var fetchPromise = fetcher.add(query);
        return fetchPromise.then(function (result) {
            dispatch(actions_1.CLAY_DATA_RESET.create.unicast(_this.uniqueId, result.data || []));
            return result.count || null;
        });
    };
    Set.prototype.sync = function (synchronizer) {
        this.appendSynchronizationModels(synchronizer);
    };
    Set.prototype.appendSynchronizationModels = function (synchronizer) {
        this.setItems.forEach(function (item) { return item.appendSynchronizationModels(synchronizer); });
    };
    Set.prototype.buildInsertModel = function () {
        return this.setItems
            .filter(function (item) { return item.status.added; })
            .map(function (item) { return item.buildInsertModel(); })
            .toArray();
    };
    Set.prototype.checkValidity = function () {
        if (this.setItems.some(function (item) { return !item.valid; })) {
            return false;
        }
        return true;
    };
    Set.prototype.cloneOverride = function (clone) {
        _super.prototype.cloneOverride.call(this, clone);
        clone.originalSet = this.originalSet;
        clone.setItems = this.setItems;
    };
    return Set;
}(StatusReporter_1.default));
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = Set;
