"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var lodash_1 = require("lodash");
var StatusReporter_1 = require("./StatusReporter");
var actions_1 = require("./actions");
var Value = (function (_super) {
    __extends(Value, _super);
    function Value(validator, synchronized, value) {
        var _this = _super.call(this, synchronized) || this;
        _this.originalValue = synchronized ? _this : null;
        _this.valueValidator = validator || null;
        _this.rawValue = value;
        _this.errorBundle = validator ? validator(value) : null;
        return _this;
    }
    Object.defineProperty(Value.prototype, "value", {
        get: function () {
            return this.rawValue;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Value.prototype, "original", {
        get: function () {
            return this.originalValue;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Value.prototype, "error", {
        get: function () {
            return this.errorBundle;
        },
        enumerable: true,
        configurable: true
    });
    Value.prototype.reduce = function (action) {
        if (actions_1.CLAY_DATA_CLEAR.is(this.uniqueId, action)) {
            if (!this.originalValue) {
                return this;
            }
            return this.originalValue;
        }
        if (actions_1.CLAY_DATA_DECAY.is(this.uniqueId, action)) {
            if (!this.statusInstance.fresh) {
                return this;
            }
            var clone = this.clone();
            clone.statusInstance = this.statusInstance.setDecayed();
            if (clone.valueValidator) {
                clone.errorBundle = clone.valueValidator(clone.value);
            }
            return clone;
        }
        if (actions_1.CLAY_DATA_SET_VALUE.is(this.uniqueId, action)) {
            if (lodash_1.isEqual(this.rawValue, action.payload)) {
                return this;
            }
            if (this.originalValue && lodash_1.isEqual(this.originalValue.value, action.payload)) {
                return this.originalValue;
            }
            var clone = this.clone();
            if (clone.valueValidator) {
                clone.errorBundle = clone.valueValidator(action.payload);
            }
            clone.rawValue = action.payload;
            if (this.statusInstance.fresh) {
                clone.statusInstance = this.statusInstance.setDecayed();
            }
            else if (this.statusInstance.synchronized && !this.statusInstance.modified) {
                clone.statusInstance = this.statusInstance.setModified();
            }
            return clone;
        }
        if (actions_1.CLAY_DATA_SET_ERROR.is(this.uniqueId, action)) {
            var clone = this.clone();
            clone.errorBundle = action.payload || null;
            return clone;
        }
        return this;
    };
    Value.prototype.clear = function (dispatch) {
        dispatch(actions_1.CLAY_DATA_CLEAR.create.unicast(this.uniqueId));
    };
    Value.prototype.decay = function (dispatch) {
        dispatch(actions_1.CLAY_DATA_DECAY.create.unicast(this.uniqueId));
    };
    Value.prototype.set = function (dispatch, value) {
        dispatch(actions_1.CLAY_DATA_SET_VALUE.create.unicast(this.uniqueId, value));
    };
    Value.prototype.setError = function (dispatch, error) {
        dispatch(actions_1.CLAY_DATA_SET_ERROR.create.unicast(this.uniqueId, error));
    };
    Value.prototype.checkValidity = function () {
        if (!this.errorBundle) {
            return true;
        }
        return false;
    };
    Value.prototype.cloneOverride = function (clone) {
        _super.prototype.cloneOverride.call(this, clone);
        clone.originalValue = this.originalValue;
        clone.valueValidator = this.valueValidator;
        clone.rawValue = this.rawValue;
        clone.errorBundle = this.errorBundle;
    };
    Value.prototype.createInstance = function () {
        return new Value(null, false, undefined);
    };
    return Value;
}(StatusReporter_1.default));
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = Value;
