"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var lodash_1 = require("lodash");
var actions_1 = require("./actions");
var UniqueReducible_1 = require("../UniqueReducible");
var Filterer = (function (_super) {
    __extends(Filterer, _super);
    function Filterer(filters) {
        var _this = _super.call(this) || this;
        if (filters) {
            _this.queryFilters = {};
            Object.keys(filters).forEach(function (key) {
                _this.queryFilters[key] = filters[key];
            });
        }
        return _this;
    }
    Object.defineProperty(Filterer.prototype, "filters", {
        get: function () {
            return this.queryFilters || {};
        },
        enumerable: true,
        configurable: true
    });
    Filterer.prototype.get = function (field) {
        if (!this.queryFilters) {
            return;
        }
        return this.queryFilters[field];
    };
    Object.defineProperty(Filterer.prototype, "plain", {
        get: function () {
            var _this = this;
            if (!this.queryFilters) {
                return null;
            }
            var keys = Object.keys(this.queryFilters);
            if (keys.length === 0) {
                return null;
            }
            var filters = {};
            keys.forEach(function (key) {
                filters[key] = _this.queryFilters[key];
            });
            return filters;
        },
        enumerable: true,
        configurable: true
    });
    Filterer.prototype.reduce = function (action) {
        if (actions_1.CLAY_QUERY_SET_FILTERS.is(this.uniqueId, action) && action.payload) {
            if (lodash_1.isEqual(this.queryFilters, action.payload)) {
                return this;
            }
            var clone = this.clone();
            clone.queryFilters = action.payload;
            return clone;
        }
        return this;
    };
    Filterer.prototype.setAll = function (dispatch, filters) {
        var _this = this;
        var newFilters = {};
        if (this.queryFilters) {
            Object.keys(this.queryFilters).forEach(function (key) {
                newFilters[key] = _this.queryFilters[key];
            });
        }
        dispatch(actions_1.CLAY_QUERY_SET_FILTERS.create.unicast(this.uniqueId, filters));
    };
    Filterer.prototype.setFilter = function (dispatch, field, filter) {
        var _this = this;
        var newFilters = {};
        if (this.queryFilters) {
            Object.keys(this.queryFilters).forEach(function (key) {
                newFilters[key] = _this.queryFilters[key];
            });
        }
        newFilters[field] = filter;
        dispatch(actions_1.CLAY_QUERY_SET_FILTERS.create.unicast(this.uniqueId, newFilters));
    };
    Filterer.prototype.clearFilter = function (dispatch, field) {
        var _this = this;
        var newFilters = {};
        if (this.queryFilters) {
            Object.keys(this.queryFilters).forEach(function (key) {
                newFilters[key] = _this.queryFilters[key];
            });
        }
        delete newFilters[field];
        dispatch(actions_1.CLAY_QUERY_SET_FILTERS.create.unicast(this.uniqueId, newFilters));
    };
    Filterer.prototype.cloneOverride = function (clone) {
        _super.prototype.cloneOverride.call(this, clone);
        clone.queryFilters = this.queryFilters;
    };
    Filterer.prototype.createInstance = function () {
        return new Filterer();
    };
    return Filterer;
}(UniqueReducible_1.default));
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = Filterer;
