"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var Value_1 = require("../data/Value");
var UniqueReducible_1 = require("../UniqueReducible");
var Includer_1 = require("./Includer");
var Filterer_1 = require("./Filterer");
var Attributor_1 = require("./Attributor");
var Paginator_1 = require("./Paginator");
var Orderer_1 = require("./Orderer");
var Querier = (function (_super) {
    __extends(Querier, _super);
    function Querier(query, shouldExtraCausePaginationReset) {
        if (shouldExtraCausePaginationReset === void 0) { shouldExtraCausePaginationReset = false; }
        var _this = _super.call(this) || this;
        if (query === null) {
            return _this;
        }
        _this.queryModel = '' + query.model;
        _this.queryById = +query.byId;
        _this.queryCount = !!query.count;
        _this.queryIncluder = new Includer_1.default(query.include);
        _this.queryFilterer = new Filterer_1.default(query.filters);
        _this.queryAttributor = new Attributor_1.default(query.attributes);
        _this.queryPaginator = new Paginator_1.default(query.pagination);
        _this.queryOrderer = new Orderer_1.default(query.order);
        _this.queryExtra = new Value_1.default(null, true, query.extra);
        _this.shouldExtraCausePaginationReset = shouldExtraCausePaginationReset;
        return _this;
    }
    Object.defineProperty(Querier.prototype, "model", {
        get: function () {
            return this.queryModel;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Querier.prototype, "byId", {
        get: function () {
            return this.queryById;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Querier.prototype, "count", {
        get: function () {
            return this.queryCount;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Querier.prototype, "includer", {
        get: function () {
            return this.queryIncluder;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Querier.prototype, "filterer", {
        get: function () {
            return this.queryFilterer;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Querier.prototype, "attributor", {
        get: function () {
            return this.queryAttributor;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Querier.prototype, "paginator", {
        get: function () {
            return this.queryPaginator;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Querier.prototype, "orderer", {
        get: function () {
            return this.queryOrderer;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Querier.prototype, "extra", {
        get: function () {
            return this.queryExtra;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Querier.prototype, "plain", {
        get: function () {
            var query = {};
            query.model = this.queryModel;
            if (typeof this.queryById === 'number') {
                query.byId = this.queryById;
            }
            var include = this.queryIncluder.plain;
            if (include !== null) {
                query.include = include;
            }
            var filters = this.queryFilterer.plain;
            if (filters !== null) {
                query.filters = filters;
            }
            var attributes = this.queryAttributor.plain;
            if (attributes !== null) {
                query.attributes = attributes;
            }
            if (this.queryCount === true) {
                query.count = true;
            }
            var pagination = this.queryPaginator.plain;
            if (pagination !== null) {
                query.pagination = pagination;
            }
            var order = this.queryOrderer.plain;
            if (order !== null) {
                query.order = order;
            }
            query.extra = this.queryExtra.value;
            return query;
        },
        enumerable: true,
        configurable: true
    });
    Querier.prototype.reduce = function (action) {
        var includer = this.queryIncluder.reduce(action);
        var filterer = this.queryFilterer.reduce(action);
        var attributor = this.queryAttributor.reduce(action);
        var paginator = this.queryPaginator.reduce(action);
        var orderer = this.queryOrderer.reduce(action);
        var extra = this.queryExtra.reduce(action);
        if (includer !== this.queryIncluder ||
            filterer !== this.queryFilterer ||
            attributor !== this.queryAttributor ||
            paginator !== this.queryPaginator ||
            orderer !== this.queryOrderer ||
            extra !== this.queryExtra) {
            var clone = this.clone();
            var shouldResetPagination = false;
            if (filterer !== this.queryFilterer ||
                attributor !== this.queryAttributor ||
                orderer !== this.queryOrderer) {
                shouldResetPagination = true;
            }
            if (this.shouldExtraCausePaginationReset && extra !== this.queryExtra) {
                shouldResetPagination = true;
            }
            if (paginator !== this.queryPaginator) {
                shouldResetPagination = false;
            }
            clone.queryIncluder = includer;
            clone.queryFilterer = filterer;
            clone.queryAttributor = attributor;
            if (shouldResetPagination) {
                clone.queryPaginator = new Paginator_1.default({ take: paginator.take });
            }
            else {
                clone.queryPaginator = paginator;
            }
            clone.queryOrderer = orderer;
            clone.queryExtra = extra;
            return clone;
        }
        return this;
    };
    Querier.prototype.cloneOverride = function (clone) {
        _super.prototype.cloneOverride.call(this, clone);
        clone.queryModel = this.queryModel;
        clone.queryById = this.queryById;
        clone.queryCount = this.count;
        clone.queryIncluder = this.queryIncluder;
        clone.queryFilterer = this.queryFilterer;
        clone.queryAttributor = this.queryAttributor;
        clone.queryPaginator = this.queryPaginator;
        clone.queryOrderer = this.queryOrderer;
        clone.queryExtra = this.queryExtra;
        clone.shouldExtraCausePaginationReset = this.shouldExtraCausePaginationReset;
    };
    Querier.prototype.createInstance = function () {
        return new Querier(null);
    };
    return Querier;
}(UniqueReducible_1.default));
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = Querier;
